import { Injectable } from '@angular/core';
import { HTTP as nativeHttp } from '@ionic-native/http';
import { HttpClient as angularHttp, HttpHeaders, HttpRequest } from "@angular/common/http";
import { checkAvailability } from '@ionic-native/core';
import { Observable } from "rxjs/Observable";
import "rxjs";
var HttpWrapper = /** @class */ (function () {
    function HttpWrapper(native, angular) {
        this.native = native;
        this.angular = angular;
        this.nativeIsAvailable = null;
        this.nativeHttp = false;
        this.angularHttp = false;
        this.nativeHttp = native;
        this.angularHttp = angular;
    }
    HttpWrapper.prototype.isNativeHttpAvailable = function () {
        if (this.nativeIsAvailable === null) {
            this.nativeIsAvailable = checkAvailability('cordova.plugin.http') === true || checkAvailability('cordovaHTTP') === true;
        }
        return this.nativeIsAvailable;
    };
    HttpWrapper.prototype.get = function (url, options) {
        var modifiedOptions = options;
        if (options) {
            modifiedOptions = options.clone({
                "method": "GET"
            });
        }
        else {
            modifiedOptions = new HttpRequest('GET', url);
        }
        return this.request(url, modifiedOptions);
    };
    HttpWrapper.prototype.post = function (url, body, options) {
        var modifiedOptions = options;
        if (options) {
            modifiedOptions = options.clone({
                "method": "POST"
            });
        }
        else {
            modifiedOptions = new HttpRequest('POST', url, body);
        }
        return this.request(url, modifiedOptions, body);
    };
    HttpWrapper.prototype.put = function (url, body, options) {
        var modifiedOptions = options;
        if (options) {
            modifiedOptions = options.clone({
                "method": "PUT"
            });
        }
        else {
            modifiedOptions = new HttpRequest('PUT', url, body);
        }
        return this.request(url, modifiedOptions, body);
    };
    HttpWrapper.prototype.delete = function (url, options) {
        var modifiedOptions = options;
        if (options) {
            modifiedOptions = options.clone({
                "method": "DELETE"
            });
        }
        else {
            modifiedOptions = new HttpRequest('DELETE', url);
        }
        return this.request(url, modifiedOptions);
    };
    HttpWrapper.prototype.request = function (url, options, data) {
        if (this.isNativeHttpAvailable()) {
            var headers = options.headers;
            if (headers instanceof Headers) {
                var newHeaders_1 = {};
                headers.forEach(function (value, name) {
                    newHeaders_1[name.toString()] = value.toString();
                });
                headers = newHeaders_1;
            }
            switch (options.method) {
                case 'GET':
                    if (data == null) {
                        data = options.params;
                    }
                    return Observable.fromPromise(this.nativeHttp.get(url, data, headers)).map(function (res) {
                        return {
                            json: function () {
                                return JSON.parse(res.data);
                            },
                            text: function (ignoredEncodingHint) {
                                return res.data.toString();
                            },
                            data: res.data,
                            headers: new Headers(res.headers)
                        };
                    });
                case 'POST':
                    return Observable.fromPromise(this.nativeHttp.post(url, data, headers)).map(function (res) {
                        return {
                            json: function () {
                                return JSON.parse(res.data);
                            },
                            text: function (ignoredEncodingHint) {
                                return res.data.toString();
                            },
                            data: res.data,
                            headers: new Headers(res.headers)
                        };
                    });
                case 'PUT':
                    if (data == null) {
                        data = options.params != null ? options.params : {};
                    }
                    return Observable.fromPromise(this.nativeHttp.put(url, data, headers)).map(function (res) {
                        return {
                            json: function () {
                                return JSON.parse(res.data);
                            },
                            text: function (ignoredEncodingHint) {
                                return res.data.toString();
                            },
                            data: res.data,
                            headers: new Headers(res.headers)
                        };
                    });
                case 'DELETE':
                    return Observable.fromPromise(this.nativeHttp.delete(url, data, headers)).map(function (res) {
                        return {
                            json: function () {
                                return JSON.parse(res.data);
                            },
                            text: function (ignoredEncodingHint) {
                                return res.data.toString();
                            },
                            data: res.data,
                            headers: new Headers(res.headers)
                        };
                    });
                default:
                    throw 'Request Method not found';
            }
        }
        else {
            //Make an @angular/common/http request
            var modifiedOptions = options.clone({
                'url': url
            });
            if (options.headers === undefined) {
                modifiedOptions = options.clone({
                    "headers": new HttpHeaders()
                });
            }
            if (data) {
                modifiedOptions = modifiedOptions.clone({
                    "body": JSON.stringify(data)
                });
            }
            return this.angularHttp.request(modifiedOptions);
        }
    };
    HttpWrapper.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    HttpWrapper.ctorParameters = function () { return [
        { type: nativeHttp, },
        { type: angularHttp, },
    ]; };
    return HttpWrapper;
}());
export { HttpWrapper };
//# sourceMappingURL=http-wrapper.js.map